import { Server } from 'socket.io'
import { TSocket } from '#utilities/types'
import { Character } from '@prisma/client'
import CharacterRepository from '#repositories/characterRepository'
import { CharacterService } from '#services/characterService'
import { ZCharacterCreate } from '#utilities/zodTypes'
import { gameLogger } from '#utilities/logger'
import { ZodError } from 'zod'

export default class CharacterCreateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('character:create', this.handleCharacterCreate.bind(this))
  }

  private async handleCharacterCreate(data: any): Promise<any> {
    // zod validate
    try {
      data = ZCharacterCreate.parse(data)

      const user_id = this.socket.userId!

      // Check if character name already exists
      const characterExists = await CharacterRepository.getByName(data.name)

      if (characterExists) {
        return this.socket.emit('notification', { message: 'Character name already exists' })
      }

      let characters: Character[] = (await CharacterRepository.getByUserId(user_id)) as Character[]

      if (characters.length >= 4) {
        return this.socket.emit('notification', { message: 'You can only have 4 characters' })
      }

      const characterService = new CharacterService()
      const character: Character = await characterService.create(data.name, user_id)

      characters = [...characters, character]

      this.socket.emit('character:create:success')
      this.socket.emit('character:list', characters)

      gameLogger.info('character:create success')
    } catch (error: any) {
      gameLogger.error(`character:create error: ${error.message}`)
      if (error instanceof ZodError) {
        return this.socket.emit('notification', { message: error.issues[0].message })
      }
      return this.socket.emit('notification', { message: 'Could not create character. Please try again (later).' })
    }
  }
}
